@@ -172,11 +172,11 @@ static PyObject* CallPythonFuncOnOneArg(PyObject* function, PyObject* single_arg
 #ifndef NMZ_RELEASE
 static_assert(
     false,
-    "Your Normaliz version (unknown) is to old! Update to 3.8.9 or newer.");
+    "Your Normaliz version (unknown) is to old! Update to 3.8.10 or newer.");
 #endif
-#if NMZ_RELEASE < 30809
+#if NMZ_RELEASE < 30810
 static_assert(false,
-              "Your Normaliz version is to old! Update to 3.8.9 or newer.");
+              "Your Normaliz version is to old! Update to 3.8.10 or newer.");
 #endif
 
 /***************************************************************************
@@ -474,6 +474,9 @@ static bool prepare_nf_input(vector< vector< NumberFieldElem > >& out,
                 current_elem = NumberFieldElem(*nf);
                 current_elem = PyUnicodeToString(current_element);
             }
+            if (PyFloat_Check(current_element)){
+                throw PyNormalizInputException("Nonintegral numbers must be given as strings"); 
+            }
             if (PyLong_Check(current_element)) {
                 mpq_class tmp;
                 current_res = PyNumberToNmz(current_element, tmp);
@@ -605,6 +608,28 @@ static PyObject* NmzWeightedEhrhartQuasiPolynomialToPyList(
 
 template < typename Integer >
 static PyObject* NmzTriangleListToPyList(
+    const pair<vector<libnormaliz::SHORTSIMPLEX<Integer> >, libnormaliz::Matrix<Integer> >& in)
+{
+    const size_t n = in.first.size();
+    PyObject*    M = PyList_New(n);
+    for (size_t i = 0; i < n; ++i) {
+        // convert the pair
+        PyObject* triple = PyList_New(3);
+        PyList_SetItem(triple, 0,
+                       NmzVectorToPyList< libnormaliz::key_t >(in.first[i].key));
+        PyList_SetItem(triple, 1, NmzToPyNumber(in.first[i].vol));
+        PyList_SetItem(triple, 2, NmzToPyNumber(libnormaliz::bool_to_bitset(in.first[i].Excluded)));
+        PyList_SetItem(M, i, triple);
+    }
+    
+    PyObject* Tr = PyList_New(2);
+    PyList_SetItem(Tr, 0,M);
+    PyList_SetItem(Tr, 1,NmzMatrixToPyList(in.second.get_elements()));
+    return Tr;
+}
+
+template < typename Integer >
+static PyObject* NmzPairVectorToPyList(
     const vector< pair< vector< libnormaliz::key_t >, Integer > >& in)
 {
     const size_t n = in.size();
@@ -634,17 +659,20 @@ NmzStanleyDataToPyList(const libnormaliz::STANLEYDATA< Integer >& StanleyData)
 
 template < typename Integer >
 static PyObject* NmzStanleyDecToPyList(
-    const std::list< libnormaliz::STANLEYDATA< Integer > >& StanleyDec)
+    const std::pair<std::list<libnormaliz::STANLEYDATA<Integer> >, libnormaliz::Matrix<Integer> > & StanleyDec)
 {
-    const size_t n = StanleyDec.size();
+    const size_t n = StanleyDec.first.size();
     PyObject*    M = PyList_New(n);
     typename std::list< libnormaliz::STANLEYDATA< Integer > >::const_iterator S =
-        StanleyDec.begin();
+        StanleyDec.first.begin();
     for (size_t i = 0; i < n; ++i) {
         PyList_SetItem(M, i, NmzStanleyDataToPyList(*S));
         ++S;
     }
-    return M;
+    PyObject*  St=PyList_New(2);
+    PyList_SetItem(St,0, M);
+    PyList_SetItem(St, 1, NmzMatrixToPyList(StanleyDec.second.get_elements()) );
+    return St;
 }
 
 template < typename Integer >
@@ -905,69 +933,13 @@ static PyObject* NmzListConeProperties(PyObject* args)
  ***************************************************************************/
 
 template < typename Integer >
-static PyObject* _NmzConeIntern(PyObject* args, PyObject* kwargs)
+static PyObject* _NmzConeIntern(PyObject* kwargs)
 {
     map< InputType, vector< vector< mpq_class > > > input;
 
-    PyObject* input_list;
-
     bool   grading_polynomial = false;
     string polynomial;
 
-    if (PyTuple_Size(args) == 1) {
-        input_list = PyTuple_GetItem(args, 0);
-        if (!PySequence_Check(input_list)) {
-            PyErr_SetString(PyNormaliz_cppError,
-                            "Single argument must be a list");
-            return NULL;
-        }
-        input_list = PyList_AsTuple(input_list);
-    }
-    else {
-        input_list = args;
-    }
-
-    const int n = PyTuple_Size(input_list);
-    if (n & 1) {
-        PyErr_SetString(PyNormaliz_cppError,
-                        "Number of arguments must be even");
-        return NULL;
-    }
-    for (int i = 0; i < n; i += 2) {
-        PyObject* type = PyTuple_GetItem(input_list, i);
-        if (!string_check(type)) {
-            PyErr_SetString(PyNormaliz_cppError,
-                            "Odd entries must be strings");
-            return NULL;
-        }
-
-        string type_str = PyUnicodeToString(type);
-
-        if (type_str.compare("polynomial") == 0) {
-            PyObject* M = PyTuple_GetItem(input_list, i + 1);
-            polynomial = PyUnicodeToString(M);
-            grading_polynomial = true;
-            continue;
-        }
-
-        PyObject* M = PyTuple_GetItem(input_list, i + 1);
-        if (M == Py_None)
-            continue;
-        vector< vector< mpq_class > > Mat;
-        try {
-            PyInputToNmz(Mat, M);
-        }
-        catch (PyNormalizInputException& e) {
-            PyErr_SetString(
-                PyNormaliz_cppError,
-                (string("When parsing ") + type_str + ": " + e.what_message())
-                    .c_str());
-            return NULL;
-        }
-
-        input[libnormaliz::to_type(type_str)] = Mat;
-    }
-
     if (kwargs != NULL) {
         PyObject* keys = PyDict_Keys(kwargs);
         PyObject* values = PyDict_Values(kwargs);
@@ -1013,7 +985,7 @@ static PyObject* _NmzConeIntern(PyObject* args, PyObject* kwargs)
 }
 
 #ifdef ENFNORMALIZ
-static PyObject* _NmzConeIntern_renf(PyObject* args, PyObject* kwargs)
+static PyObject* _NmzConeIntern_renf(PyObject* kwargs)
 {
 
     FUNC_BEGIN
@@ -1037,7 +1009,7 @@ static PyObject* _NmzConeIntern_renf(PyObject* args, PyObject* kwargs)
 
 
     renf_class* renf;
-    // number_field_data contains 4 entries: poly, var, emb
+    // number_field_data contains 3 entries: poly, var, emb
     // All are strings
     string poly = PyUnicodeToString(PySequence_GetItem(number_field_data, 0));
     string var = PyUnicodeToString(PySequence_GetItem(number_field_data, 1));
@@ -1113,15 +1085,15 @@ static PyObject* _NmzCone(PyObject* self, PyObject* args, PyObject* kwargs)
     if (kwargs != NULL && PyDict_Contains(kwargs, create_as_long_long) == 1) {
         create_as_long_long = PyDict_GetItem(kwargs, create_as_long_long);
         if (create_as_long_long == Py_True) {
-            return _NmzConeIntern< long long >(args, kwargs);
+            return _NmzConeIntern< long long >(kwargs);
         }
     }
 #ifdef ENFNORMALIZ
     else if (kwargs != NULL && PyDict_Contains(kwargs, create_as_renf) == 1) {
-        return _NmzConeIntern_renf(args, kwargs);
+        return _NmzConeIntern_renf(kwargs);
     }
 #endif
-    return _NmzConeIntern< mpz_class >(args, kwargs);
+    return _NmzConeIntern< mpz_class >(kwargs);
     FUNC_END
 }
 
@@ -1168,6 +1140,7 @@ static PyObject* _NmzConeCopy(PyObject* self, PyObject* args)
  *
  ***************************************************************************/
 
+/* SUPERFLUOUS
 template < typename Integer >
 static PyObject* NmzHilbertSeries(Cone< Integer >* C, PyObject* args)
 {
@@ -1223,6 +1196,7 @@ static PyObject* NmzHilbertSeries_Outer(PyObject* self, PyObject* args)
     }
     FUNC_END
 }
+*/
 
 /***************************************************************************
  *
@@ -1458,19 +1432,35 @@ static PyObject* NmzIsComputed_Outer(PyObject* self, PyObject* args)
  ***************************************************************************/
 
 template < typename Integer >
-static PyObject* NmzSetGrading_inner(Cone< Integer >* cone, PyObject* list)
+static PyObject* NmzSetGrading_inner(Cone< Integer >* cone, PyObject* grad)
 {
-    vector< Integer > list_c;
-    bool              result = PyListToNmz(list_c, list);
+    vector< Integer > grad_c;
+    bool result = PyListToNmz(grad_c, grad);
     if (!result) {
         PyErr_SetString(PyNormaliz_cppError,
                         "grading argument is not an integer list");
         return NULL;
     }
-    cone->resetGrading(list_c);
+    cone->resetGrading(grad_c);
     Py_RETURN_NONE;
 }
 
+#ifdef ENFNORMALIZ
+template <>
+PyObject* NmzSetGrading_inner(Cone< renf_elem_class >* cone, PyObject* grad)
+{
+    vector< renf_elem_class > grad_renf;
+    vector<vector< renf_elem_class> > grad_mat; // a cheap way to convert vectors
+    PyObject*  PyHelpMat = PyList_New(1);     // better: rebuild conversion to renf
+    PyList_SetItem(PyHelpMat, 0, grad);    
+    prepare_nf_input(grad_mat, PyHelpMat,cone->getRenf());
+    grad_renf = grad_mat[0];
+
+    cone->resetGrading(grad_renf);
+    Py_RETURN_NONE;
+}
+#endif
+
 static PyObject* NmzSetGrading(PyObject* self, PyObject* args)
 {
     FUNC_BEGIN
@@ -1484,11 +1474,88 @@ static PyObject* NmzSetGrading(PyObject* self, PyObject* args)
         Cone< mpz_class >* cone_ptr = get_cone_mpz(cone);
         return NmzSetGrading_inner(cone_ptr, grading_py);
     }
-    else {
+     if (is_cone_long(cone)) {
         Cone< long long >* cone_ptr = get_cone_long(cone);
         return NmzSetGrading_inner(cone_ptr, grading_py);
     }
+#ifdef ENFNORMALIZ
+     if (is_cone_renf(cone)) {
+        Cone< renf_elem_class >* cone_ptr = get_cone_renf(cone);
+        return NmzSetGrading_inner(cone_ptr, grading_py);
+    }
+#endif
     FUNC_END
+    
+    Py_RETURN_NONE;
+}
+
+/***************************************************************************
+ *
+ * NmzSetProjectionCoords
+ *
+ ***************************************************************************/
+
+template < typename Integer >
+static PyObject* NmzSetProjectionCoords_inner(Cone< Integer >* cone, PyObject* coords)
+{
+    vector< Integer > coords_c;
+    bool  result = PyListToNmz(coords_c, coords);
+    if (!result) {
+        PyErr_SetString(PyNormaliz_cppError,
+                        " is not an integer list");
+        return NULL;
+    }
+    for(size_t i=0; i< coords_c.size(); ++i){
+        if(coords_c[i]!=0 && coords_c[i]!=1)
+            PyErr_SetString(PyNormaliz_cppError, "Projection coordinates must be 0 or 1");
+    }
+            
+    cone->resetProjectionCoords(coords_c);
+    Py_RETURN_NONE;
+}
+
+#ifdef ENFNORMALIZ
+template <>
+PyObject* NmzSetProjectionCoords_inner(Cone< renf_elem_class >* cone, PyObject* coords)
+{
+    vector< renf_elem_class > coords_renf;
+    vector<vector< renf_elem_class> > coords_mat; // a cheap way to convert vectors
+    PyObject*  PyHelpMat = PyList_New(1);     // better: rebuild conversion to renf
+    PyList_SetItem(PyHelpMat, 0, coords);    
+    prepare_nf_input(coords_mat, PyHelpMat,cone->getRenf());
+    coords_renf = coords_mat[0];
+
+    cone->resetGrading(coords_renf);
+    Py_RETURN_NONE;
+}
+#endif
+
+
+static PyObject* NmzSetProjectionCoords(PyObject* self, PyObject* args)
+{
+    FUNC_BEGIN
+    PyObject* cone = PyTuple_GetItem(args, 0);
+    PyObject* coords_py = PyTuple_GetItem(args, 1);
+    if (!is_cone(cone)) {
+        PyErr_SetString(PyNormaliz_cppError, "First argument must be a cone");
+        return NULL;
+    }
+    if (is_cone_long(cone)) {
+        Cone< long long >* cone_ptr = get_cone_long(cone);
+        return NmzSetProjectionCoords_inner(cone_ptr, coords_py);
+    }
+    if (is_cone_mpz(cone)) {
+        Cone< mpz_class >* cone_ptr = get_cone_mpz(cone);
+        return NmzSetProjectionCoords_inner(cone_ptr, coords_py);
+    }
+#ifdef ENFNORMALIZ
+    if (is_cone_renf(cone)) {
+        Cone< renf_elem_class >* cone_ptr = get_cone_renf(cone);
+        return NmzSetProjectionCoords_inner(cone_ptr, coords_py);
+    }
+#endif
+    FUNC_END
+    Py_RETURN_NONE;
 }
 
 /***************************************************************************
@@ -1544,11 +1611,15 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
     // Handle standard cases
     libnormaliz::OutputType::Enum outputtype = libnormaliz::output_type(p);
 
-
     switch (p) {
-            
-        case libnormaliz::ConeProperty::Triangulation:
+                  
+        case libnormaliz::ConeProperty::Triangulation:{
             return NmzTriangleListToPyList< Integer >(C->getTriangulation());
+        }
+        
+        case libnormaliz::ConeProperty::ConeDecomposition:{
+            return NmzTriangleListToPyList< Integer >(C->getConeDecomposition());
+        }
             
         case libnormaliz::ConeProperty::AllGeneratorsTriangulation:
             return NmzTriangleListToPyList< Integer >(C->getTriangulation(
@@ -1573,8 +1644,7 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
         }
 
         case libnormaliz::ConeProperty::WeightedEhrhartSeries:
-            return NmzWeightedEhrhartSeriesToPyList(
-                C->getWeightedEhrhartSeries());
+            return NmzWeightedEhrhartSeriesToPyList( C->getWeightedEhrhartSeries());
 
         // though Grading has the return type vector<Integer> we make it
         // a complex struture within PyNormaliz since we want to combine it
@@ -1592,7 +1662,7 @@ _NmzResultImpl(Cone< Integer >* C, PyObject* prop_obj, void* nf = nullptr)
             return NmzStanleyDecToPyList(C->getStanleyDec());
 
         case libnormaliz::ConeProperty::InclusionExclusionData:
-            return NmzTriangleListToPyList< long >(
+            return NmzPairVectorToPyList< long >(
                 C->getInclusionExclusionData());
 
         /* returned as a matrix, no need to make it a complex property
@@ -1930,6 +2000,100 @@ static PyObject* NmzSetNrCoeffQuasiPol(PyObject* self, PyObject* args)
     FUNC_END
 }
 
+/***************************************************************************
+ *
+ * Polynomial
+ *
+ ***************************************************************************/
+
+static PyObject* NmzSetPolynomial(PyObject* self, PyObject* args)
+{
+
+    FUNC_BEGIN
+
+    PyObject* cone = PyTuple_GetItem(args, 0);
+
+    if (!is_cone(cone)) {
+        PyErr_SetString(PyNormaliz_cppError, "First argument must be a cone");
+        return NULL;
+    }
+    
+    TempSignalHandler tmpHandler; // use custom signal handler
+
+    PyObject* poly_pi = PyTuple_GetItem(args, 1);
+    
+    if(!string_check(poly_pi)){
+        PyErr_SetString(PyNormaliz_cppError, "Polynomual must be given as a string");
+        return NULL;
+    }
+    TempSignalHandler tmpHandler1; // use custom signal handler
+    
+    string polynomial = PyUnicodeToString(poly_pi);
+    
+    if (is_cone_mpz(cone)) {
+        Cone< mpz_class >* cone_ptr = get_cone_mpz(cone);
+        cone_ptr->setPolynomial(polynomial);
+        Py_RETURN_TRUE;
+    }
+    else if (is_cone_long(cone)) {
+        Cone< long long >* cone_ptr = get_cone_long(cone);
+        cone_ptr->setPolynomial(polynomial);
+        Py_RETURN_TRUE;
+    }
+    else {
+        PyErr_SetString(PyNormaliz_cppError,
+                        "Polynomial cannot be set for renf cone");
+        return NULL;
+    }
+
+    FUNC_END
+}
+
+/***************************************************************************
+ *
+ * FaceCodimBound
+ *
+ ***************************************************************************/
+
+static PyObject* NmzSetFaceCodimBound(PyObject* self, PyObject* args)
+{
+
+    FUNC_BEGIN
+
+    PyObject* cone = PyTuple_GetItem(args, 0);
+
+    if (!is_cone(cone)) {
+        PyErr_SetString(PyNormaliz_cppError, "First argument must be a cone");
+        return NULL;
+    }
+
+    PyObject* bound_py = PyTuple_GetItem(args, 1);
+
+    TempSignalHandler tmpHandler; // use custom signal handler
+
+    int  overflow;
+    long bound = PyLong_AsLongLongAndOverflow(bound_py, &overflow);
+    if (is_cone_mpz(cone)) {
+        Cone< mpz_class >* cone_ptr = get_cone_mpz(cone);
+        cone_ptr->setFaceCodimBound(bound);
+        Py_RETURN_TRUE;
+    }
+    else if (is_cone_long(cone)) {
+        Cone< long long >* cone_ptr = get_cone_long(cone);
+        cone_ptr->setFaceCodimBound(bound);
+        Py_RETURN_TRUE;
+    }
+#ifdef ENFNORMALIZ
+    else {
+         Cone<renf_elem_class>* cone_ptr = get_cone_renf(cone);
+         cone_ptr->setFaceCodimBound(bound);
+         Py_RETURN_TRUE;
+    }
+#endif
+
+    FUNC_END
+}
+
 /***************************************************************************
  *
  * Get Symmetrized cone
@@ -2059,8 +2223,12 @@ static PyObject* NmzGetWeightedEhrhartSeriesExpansion(PyObject* self, PyObject*
     }
 
     ES.first.set_expansion_degree(degree);
-    return NmzVectorToPyList(ES.first.getExpansion());
 
+    PyObject* return_list = PyList_New(2);
+    PyList_SetItem(return_list, 0, NmzVectorToPyList(ES.first.getExpansion()));
+        PyList_SetItem(return_list, 1, NmzToPyNumber(ES.second));
+    return return_list;
+    
     FUNC_END
 }
 
@@ -2252,12 +2420,12 @@ static PyObject* NmzGetRenfInfo(PyObject* self, PyObject* args)
         );
         return NULL;
     }
-
     renf_class* renf = get_cone_renf_renf(cone_py);
-    double a_double = renf->gen().get_d();
+    std::string minpoly_str;
+    minpoly_str = fmpq_poly_get_str_pretty(renf->get_renf()->nf->pol, renf->gen_name.c_str());
     std::string res1 = arb_get_str(renf->get_renf()->emb, 64, 0);
-    long prec = renf->get_renf()->prec;
-    return PyTuple_Pack(3,PyFloat_FromDouble(a_double),StringToPyUnicode(res1),PyLong_FromLong(prec));
+    // long prec = renf->get_renf()->prec;
+    return PyTuple_Pack(2, StringToPyUnicode(minpoly_str), StringToPyUnicode(res1));
 #else
     return NULL;
 #endif
@@ -2301,6 +2469,8 @@ static PyMethodDef PyNormaliz_cppMethods[] = {
      "Check if property is computed "},
     {"NmzSetGrading", (PyCFunction)NmzSetGrading, METH_VARARGS,
      "Reset the grading of a cone"},
+    {"NmzSetProjectionCoords", (PyCFunction)NmzSetProjectionCoords, METH_VARARGS,
+     "Reset the projection coordinates"},
     {"NmzResult", (PyCFunction)_NmzResult, METH_VARARGS | METH_KEYWORDS,
      "Return cone property"},
     {"NmzSetVerboseDefault", (PyCFunction)NmzSetVerboseDefault, METH_VARARGS,
@@ -2309,8 +2479,8 @@ static PyMethodDef PyNormaliz_cppMethods[] = {
      "Set verbosity of cone"},
     {"NmzListConeProperties", (PyCFunction)NmzListConeProperties, METH_NOARGS,
      "List all available properties"},
-    {"NmzHilbertSeries", (PyCFunction)NmzHilbertSeries_Outer, METH_VARARGS,
-     "Returns Hilbert series, either HSOP or not"},
+    // {"NmzHilbertSeries", (PyCFunction)NmzHilbertSeries_Outer, METH_VARARGS,
+    // "Returns Hilbert series, either HSOP or not"},
     {"NmzGetPolynomial", (PyCFunction)NmzGetPolynomial, METH_VARARGS,
      "Returns grading polynomial"},
     {"NmzSymmetrizedCone", (PyCFunction)NmzSymmetrizedCone, METH_VARARGS,
@@ -2319,7 +2489,11 @@ static PyMethodDef PyNormaliz_cppMethods[] = {
      (PyCFunction)NmzSetNumberOfNormalizThreads, METH_VARARGS,
      "Sets the Normaliz thread limit"},
     {"NmzSetNrCoeffQuasiPol", (PyCFunction)NmzSetNrCoeffQuasiPol,
-     METH_VARARGS, "Sets the period bound for the quasi-polynomial"},
+     METH_VARARGS, "Sets the number of computed coefficients for the quasi-polynomial"},
+    {"NmzSetPolynomial", (PyCFunction)NmzSetPolynomial,
+     METH_VARARGS, "Sets the polynomial for integration and weighted series"},
+    {"NmzSetFaceCodimBound", (PyCFunction)NmzSetFaceCodimBound,
+     METH_VARARGS, "Sets the maximal codimension for the computed faces"},
     {"NmzGetHilbertSeriesExpansion",
      (PyCFunction)NmzGetHilbertSeriesExpansion, METH_VARARGS,
      "Returns expansion of the hilbert series"},
